@@ -356,8 +356,10 @@ mozilla_embed_init (MozillaEmbed *embed)
 }
 
 gpointer
-mozilla_embed_get_galeon_wrapper (MozillaEmbed *embed)
+mozilla_embed_get_ephy_wrapper (MozillaEmbed *embed)
 {
+	g_return_val_if_fail (embed->priv->wrapper != NULL, NULL);
+	
 	return embed->priv->wrapper;
 }
 
@@ -422,14 +424,6 @@ mozilla_embed_finalize (GObject *object)
 	LOG ("MozillaEmbed finalized %p", embed)
 }
 
-gpointer
-mozilla_embed_get_ephy_wrapper (MozillaEmbed *embed)
-{
-	g_return_val_if_fail (embed->priv->wrapper != NULL, NULL);
-	
-	return embed->priv->wrapper;
-}
-
 static void
 impl_get_capabilities (EphyEmbed *embed,
                        EmbedCapabilities *caps)
@@ -1064,13 +1058,7 @@ impl_get_security_level (EphyEmbed *embed,
 
 		if (tooltip)
 		{
-			const nsString &string = nsString(tooltip);
-			char *tmp;
-			tmp = ToNewCString (string);
-			
-			*description = g_strdup (tmp);
-
-			nsMemory::Free (tmp);
+			*description = g_strdup (NS_ConvertUCS2toUTF8(tooltip).get());
 		}
 	}
 	
@@ -1284,7 +1272,7 @@ mozilla_embed_link_message_cb (GtkMozEmbed *embed,
 	*getter_Copies(message) = gtk_moz_embed_get_link_message_unichar (embed);
 	
 	g_signal_emit_by_name (membed, "ge_link_message",
-			       g_strdup(NS_ConvertUCS2toUTF8(message).get()));
+			       NS_ConvertUCS2toUTF8(message).get());
 }
 
 static void
@@ -1296,7 +1284,7 @@ mozilla_embed_js_status_cb (GtkMozEmbed *embed,
 	*getter_Copies(status) = gtk_moz_embed_get_js_status_unichar (embed);
 	
 	g_signal_emit_by_name (membed, "ge_js_status",
-			       g_strdup(NS_ConvertUCS2toUTF8(status).get()));
+			       NS_ConvertUCS2toUTF8(status).get());
 }
 
 static void
@@ -1429,7 +1417,7 @@ mozilla_embed_new_window_cb (GtkMozEmbed *embed,
 		{ GTK_MOZ_EMBED_FLAG_DEFAULTCHROME, EMBED_CHROME_DEFAULT },
 		{ GTK_MOZ_EMBED_FLAG_MENUBARON, EMBED_CHROME_MENUBARON },
 		{ GTK_MOZ_EMBED_FLAG_TOOLBARON, EMBED_CHROME_TOOLBARON },
-		{ GTK_MOZ_EMBED_FLAG_PERSONALTOOLBARON, EMBED_CHROME_PERSONALTOOLBARON },
+		{ GTK_MOZ_EMBED_FLAG_PERSONALTOOLBARON, EMBED_CHROME_BOOKMARKSBARON },
 		{ GTK_MOZ_EMBED_FLAG_STATUSBARON, EMBED_CHROME_STATUSBARON },
 		{ GTK_MOZ_EMBED_FLAG_WINDOWRAISED, EMBED_CHROME_WINDOWRAISED },
 		{ GTK_MOZ_EMBED_FLAG_WINDOWLOWERED, EMBED_CHROME_WINDOWLOWERED },
