@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node.c,v 1.4 2003/04/13 10:12:32 mpeseng Exp $
+ *  $Id: ephy-node.c,v 1.7 2003/04/27 18:38:27 mpeseng Exp $
  */
 
 #include <config.h>
@@ -93,6 +93,7 @@ enum
 	CHILD_ADDED,
 	CHILD_CHANGED,
 	CHILD_REMOVED,
+	CHILDREN_REORDERED,
 	LAST_SIGNAL
 };
 
@@ -202,6 +203,16 @@ ephy_node_class_init (EphyNodeClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      EPHY_TYPE_NODE);
+	ephy_node_signals[CHILDREN_REORDERED] =
+		g_signal_new ("children_reordered",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EphyNodeClass, children_reordered),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_POINTER);
 }
 
 static gboolean
@@ -438,8 +449,6 @@ ephy_node_get_from_id (gulong id)
 {
 	EphyNode *ret = NULL;
 
-	g_return_val_if_fail (id > 0, NULL);
-
 	g_static_rw_lock_reader_lock (id_to_node_lock);
 
 	ret = node_from_id_real (id);
@@ -1146,14 +1155,6 @@ real_remove_child (EphyNode *node,
 {
 	EphyNodeParent *node_info;
 	
-	write_lock_to_read_lock (node);
-	write_lock_to_read_lock (child);
-
-	g_signal_emit (G_OBJECT (node), ephy_node_signals[CHILD_REMOVED], 0, child);
-
-	read_lock_to_write_lock (node);
-	read_lock_to_write_lock (child);
-
 	node_info = g_hash_table_lookup (child->priv->parents,
 			                 GINT_TO_POINTER (node->priv->id));
 
@@ -1184,6 +1185,15 @@ real_remove_child (EphyNode *node,
 		g_hash_table_remove (child->priv->parents,
 				     GINT_TO_POINTER (node->priv->id));
 	}
+
+	write_lock_to_read_lock (node);
+	write_lock_to_read_lock (child);
+
+	g_signal_emit (G_OBJECT (node), ephy_node_signals[CHILD_REMOVED], 0, child);
+
+	read_lock_to_write_lock (node);
+	read_lock_to_write_lock (child);
+
 }
 
 void
@@ -1227,6 +1237,48 @@ ephy_node_has_child (EphyNode *node,
 	return ret;
 }
 
+void
+ephy_node_reorder_children (EphyNode *node,
+ 			    int *new_order)
+{
+	GPtrArray *newkids;
+	int i;
+
+	g_return_if_fail (EPHY_IS_NODE (node));
+	g_return_if_fail (new_order != NULL);
+
+	lock_gdk ();
+
+	g_static_rw_lock_writer_lock (node->priv->lock);
+
+	newkids = g_ptr_array_new ();
+	g_ptr_array_set_size (newkids, node->priv->children->len);
+
+	for (i = 0; i < node->priv->children->len; i++) {
+		EphyNode *child;
+		EphyNodeParent *node_info;
+
+		child = g_ptr_array_index (node->priv->children, i);
+
+		g_ptr_array_index (newkids, new_order[i]) = child;
+
+		node_info = g_hash_table_lookup (child->priv->parents,
+					         GINT_TO_POINTER (node->priv->id));
+		node_info->index = new_order[i];
+	}
+
+	g_ptr_array_free (node->priv->children, FALSE);
+	node->priv->children = newkids;
+
+	write_lock_to_read_lock (node);
+
+	g_signal_emit (G_OBJECT (node), ephy_node_signals[CHILDREN_REORDERED], 0, new_order);
+
+	g_static_rw_lock_reader_unlock (node->priv->lock);
+
+	unlock_gdk ();
+}
+
 GPtrArray *
 ephy_node_get_children (EphyNode *node)
 {
