@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node.h,v 1.3 2003/04/13 10:12:32 mpeseng Exp $
+ *  $Id: ephy-node.h,v 1.4 2003/04/25 16:06:18 xan Exp $
  */
 
 #ifndef EPHY_NODE_H
@@ -53,8 +53,7 @@ typedef struct
 
 	void (*child_added)     (EphyNode *node, EphyNode *child);
 	void (*child_changed)   (EphyNode *node, EphyNode *child);
-	void (*child_reordered) (EphyNode *node, EphyNode *child,
-			         int old_index, int new_index);
+	void (*children_reordered) (EphyNode *node, int *new_order);
 	void (*child_removed)   (EphyNode *node, EphyNode *child);
 } EphyNodeClass;
 
@@ -122,6 +121,9 @@ void        ephy_node_remove_child          (EphyNode *node,
 gboolean    ephy_node_has_child             (EphyNode *node,
 					     EphyNode *child);
 
+void	    ephy_node_reorder_children	    (EphyNode *node,
+					     int *new_order);
+
 /* Note that ephy_node_get_children freezes the node; you'll have to thaw it when done.
  * This is to prevent the data getting changed from another thread. */
 GPtrArray    *ephy_node_get_children        (EphyNode *node);
