@@ -27,7 +27,6 @@
 #include "ppview-toolbar.h"
 #include "window-commands.h"
 #include "find-dialog.h"
-#include "history-dialog.h"
 #include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
@@ -130,6 +129,9 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	{ "ViewToolbar", N_("_Toolbar"), NULL, "<shift><control>T",
 	  N_("Show or hide toolbar"),
 	  G_CALLBACK (window_cmd_view_toolbar), NULL, TOGGLE_ACTION },
+	{ "ViewBookmarksBar", N_("_Bookmarks Bar"), NULL, NULL,
+	  N_("Show or hide bookmarks bar"),
+	  G_CALLBACK (window_cmd_view_bookmarks_bar), NULL, TOGGLE_ACTION },
 	{ "ViewStatusbar", N_("St_atusbar"), NULL, NULL,
 	  N_("Show or hide statusbar"),
 	  G_CALLBACK (window_cmd_view_statusbar), NULL, TOGGLE_ACTION },
@@ -254,7 +256,6 @@ struct EphyWindowPrivate
 	GtkWidget *menu_dock;
 	GtkWidget *menubar;
 	Toolbar *toolbar;
-	GList *toolbars;
 	GtkWidget *statusbar;
 	EggActionGroup *action_group;
 	EggActionGroup *popups_action_group;
@@ -263,10 +264,7 @@ struct EphyWindowPrivate
 	PPViewToolbar *ppview_toolbar;
 	GtkNotebook *notebook;
 	EphyTab *active_tab;
-	GtkWidget *sidebar;
 	EphyDialog *find_dialog;
-	EphyDialog *history_dialog;
-	EphyDialog *history_sidebar;
 	EmbedChromeMask chrome_mask;
 	gboolean closing;
 };
@@ -316,7 +314,6 @@ ephy_window_get_type (void)
 							   "EphyWindow",
 							   &our_info, 0);
         }
-
         return ephy_window_type;
 }
 
@@ -394,21 +391,73 @@ add_widget (EggMenuMerge *merge, GtkWidget *widget, EphyWindow *window)
 	{
 		window->priv->menubar = widget;
 	}
-	else
-	{
-		window->priv->toolbars = g_list_append
-			(window->priv->toolbars, widget);
-	}
 
 	gtk_box_pack_start (GTK_BOX (window->priv->menu_dock),
 			    widget, FALSE, FALSE, 0);
 }
 
+static void
+menu_activate_cb (GtkWidget *widget,
+	          EphyWindow *window)
+{
+	gboolean cut, copy, paste, select_all;
+	EggActionGroup *action_group;
+	EggAction *action;
+	GtkWidget *focus_widget;
+	EphyEmbed *embed;
+
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
+
+	focus_widget = gtk_window_get_focus (GTK_WINDOW (window));
+
+
+	if (GTK_IS_EDITABLE (focus_widget))
+	{
+		gboolean has_selection;
+
+		has_selection = gtk_editable_get_selection_bounds
+			(GTK_EDITABLE (focus_widget), NULL, NULL);
+		paste = gtk_clipboard_wait_is_text_available
+			(gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
+		select_all = TRUE;
+		cut = has_selection;
+		copy = has_selection;
+	}
+	else if (focus_widget == GTK_WIDGET (embed) ||
+		 gtk_widget_get_ancestor (GTK_WIDGET (embed), EPHY_EMBED_TYPE))
+	{
+		paste = (ephy_embed_can_paste (embed) == G_OK);
+		cut = (ephy_embed_selection_can_cut (embed) == G_OK);
+		copy = (ephy_embed_selection_can_copy (embed) == G_OK);
+		select_all = TRUE;
+	}
+	else
+	{
+		paste = FALSE;
+		cut = FALSE;
+		copy = FALSE;
+		select_all = FALSE;
+	}
+
+	action_group = window->priv->action_group;
+	action = egg_action_group_get_action (action_group, "EditCut");
+	g_object_set (action, "sensitive", cut, NULL);
+	action = egg_action_group_get_action (action_group, "EditCopy");
+	g_object_set (action, "sensitive", copy, NULL);
+	action = egg_action_group_get_action (action_group, "EditPaste");
+	g_object_set (action, "sensitive", paste, NULL);
+	action = egg_action_group_get_action (action_group, "EditSelectAll");
+	g_object_set (action, "sensitive", select_all, NULL);
+}
+
 static void
 setup_window (EphyWindow *window)
 {
 	EggActionGroup *action_group;
+	EggAction *action;
 	EggMenuMerge *merge;
+	GtkWidget *menu;
 	int i;
 
 	window->priv->main_vbox = gtk_vbox_new (FALSE, 0);
@@ -439,6 +488,16 @@ setup_window (EphyWindow *window)
 				      ephy_menu_n_entries);
 	egg_menu_merge_insert_action_group (merge, action_group, 0);
 	window->priv->action_group = action_group;
+	action = egg_action_group_get_action (action_group, "FileOpen");
+	g_object_set (action, "short_label", _("Open"), NULL);
+	action = egg_action_group_get_action (action_group, "FileSaveAs");
+	g_object_set (action, "short_label", _("Save As"), NULL);
+	action = egg_action_group_get_action (action_group, "FilePrint");
+	g_object_set (action, "short_label", _("Print"), NULL);
+	action = egg_action_group_get_action (action_group, "FileBookmarkPage");
+	g_object_set (action, "short_label", _("Bookmark"), NULL);
+	action = egg_action_group_get_action (action_group, "ViewZoomNormal");
+	g_object_set (action, "sensitive", FALSE, NULL);
 
 	action_group = egg_action_group_new ("PopupsActions");
 	egg_action_group_add_actions (action_group, ephy_popups_entries,
@@ -451,8 +510,16 @@ setup_window (EphyWindow *window)
 	egg_menu_merge_add_ui_from_file
 		(merge, ephy_file ("epiphany-ui.xml"), NULL);
 	gtk_window_add_accel_group (GTK_WINDOW (window), merge->accel_group);
+	egg_menu_merge_ensure_update (merge);
+
+	menu = egg_menu_merge_get_widget (merge, "/menu/EditMenu");
+	g_signal_connect (menu, "activate", G_CALLBACK (menu_activate_cb), window);
 
 	window->priv->toolbar = toolbar_new (window);
+	gtk_widget_show (GTK_WIDGET (window->priv->toolbar));
+	gtk_box_pack_start (GTK_BOX (window->priv->menu_dock),
+			    GTK_WIDGET (window->priv->toolbar),
+			    FALSE, FALSE, 0);
 
 	g_signal_connect(window,
 			 "key-press-event",
@@ -500,7 +567,6 @@ ephy_window_init (EphyWindow *window)
 	window->priv->chrome_mask = 0;
 	window->priv->closing = FALSE;
 	window->priv->ppview_toolbar = NULL;
-	window->priv->toolbars = NULL;
 
 	/* Setup the window and connect verbs */
 	setup_window (window);
@@ -537,15 +603,10 @@ save_window_chrome (EphyWindow *window)
 	else if (flags & EMBED_CHROME_PPVIEWTOOLBARON)
 	{
 	}
-	else if (flags & EMBED_CHROME_OPENASFULLSCREEN)
-	{
-		eel_gconf_set_boolean (CONF_WINDOWS_FS_SHOW_TOOLBARS,
-				       flags & EMBED_CHROME_TOOLBARON);
-		eel_gconf_set_boolean (CONF_WINDOWS_FS_SHOW_STATUSBAR,
-				       flags & EMBED_CHROME_STATUSBARON);
-	}
 	else
 	{
+		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR,
+				       flags & EMBED_CHROME_BOOKMARKSBARON);
 		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_TOOLBARS,
 				       flags & EMBED_CHROME_TOOLBARON);
 		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_STATUSBAR,
@@ -582,31 +643,14 @@ ephy_window_finalize (GObject *object)
 		g_object_unref (G_OBJECT (window->priv->find_dialog));
 	}
 
-	if (window->priv->history_dialog)
-	{
-		g_object_remove_weak_pointer
-                        (G_OBJECT(window->priv->history_dialog),
-                         (gpointer *)&window->priv->history_dialog);
-	}
-
 	g_object_unref (window->priv->fav_menu);
 	g_object_unref (window->priv->enc_menu);
 
-	if (window->priv->toolbar)
-	{
-		g_object_unref (window->priv->toolbar);
-	}
-
 	if (window->priv->ppview_toolbar)
 	{
 		g_object_unref (window->priv->ppview_toolbar);
 	}
 
-	if (window->priv->toolbars)
-	{
-		g_list_free (window->priv->toolbars);
-	}
-
 	g_object_unref (window->priv->action_group);
 	egg_menu_merge_remove_action_group (EGG_MENU_MERGE (window->ui_merge),
 					    window->priv->action_group);
@@ -697,29 +741,22 @@ translate_default_chrome (EmbedChromeMask *chrome_mask)
 			 EMBED_CHROME_OPENASFULLSCREEN);
 
 	/* Load defaults */
-	if (*chrome_mask & EMBED_CHROME_OPENASFULLSCREEN)
+	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR))
 	{
-		if (eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_STATUSBAR))
-		{
-			*chrome_mask |= EMBED_CHROME_STATUSBARON;
-		}
-		if (eel_gconf_get_boolean (CONF_WINDOWS_FS_SHOW_TOOLBARS))
-		{
-			*chrome_mask |= EMBED_CHROME_TOOLBARON;
-		}
+		*chrome_mask |= EMBED_CHROME_STATUSBARON;
 	}
-	else
+	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS))
 	{
-		if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR))
-		{
-			*chrome_mask |= EMBED_CHROME_STATUSBARON;
-		}
-		if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_TOOLBARS))
-		{
-			*chrome_mask |= EMBED_CHROME_TOOLBARON;
-		}
+		*chrome_mask |= EMBED_CHROME_TOOLBARON;
+	}
+	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR))
+	{
+		*chrome_mask |= EMBED_CHROME_BOOKMARKSBARON;
+	}
 
-		*chrome_mask |= EMBED_CHROME_PERSONALTOOLBARON;
+	// Show the menu bar if we're not in fullscreen
+	if (!(*chrome_mask & EMBED_CHROME_OPENASFULLSCREEN))
+	{
 		*chrome_mask |= EMBED_CHROME_MENUBARON;
 	}
 }
@@ -735,6 +772,10 @@ update_layout_toggles (EphyWindow *window)
 	egg_toggle_action_set_active (EGG_TOGGLE_ACTION (action),
 				      mask & EMBED_CHROME_TOOLBARON);
 
+	action = egg_action_group_get_action (action_group, "ViewBookmarksBar");
+	egg_toggle_action_set_active (EGG_TOGGLE_ACTION (action),
+				      mask & EMBED_CHROME_BOOKMARKSBARON);
+
 	action = egg_action_group_get_action (action_group, "ViewStatusbar");
 	egg_toggle_action_set_active (EGG_TOGGLE_ACTION (action),
 				      mask & EMBED_CHROME_STATUSBARON);
@@ -762,16 +803,9 @@ ephy_window_set_chrome (EphyWindow *window,
 		gtk_widget_hide (window->priv->menubar);
 	}
 
-	if (flags & EMBED_CHROME_TOOLBARON)
-	{
-		g_list_foreach (window->priv->toolbars,
-				(GFunc)gtk_widget_show, NULL);
-	}
-	else
-	{
-		g_list_foreach (window->priv->toolbars,
-				(GFunc)gtk_widget_hide, NULL);
-	}
+	toolbar_set_visibility (window->priv->toolbar,
+				flags & EMBED_CHROME_TOOLBARON,
+				flags & EMBED_CHROME_BOOKMARKSBARON);
 
 	if (flags & EMBED_CHROME_STATUSBARON)
 	{
@@ -822,10 +856,12 @@ ephy_window_get_notebook (EphyWindow *window)
 
 void
 ephy_window_add_tab (EphyWindow *window,
-		       EphyTab *tab,
-		       gboolean jump_to)
+		     EphyTab *tab,
+		     gboolean grouped,
+		     gboolean jump_to)
 {
 	GtkWidget *widget;
+	int position;
 
 	g_return_if_fail (IS_EPHY_WINDOW (window));
 	g_return_if_fail (IS_EPHY_TAB (tab));
@@ -834,10 +870,9 @@ ephy_window_add_tab (EphyWindow *window,
 
 	widget = GTK_WIDGET(ephy_tab_get_embed (tab));
 
+	position = grouped ? EPHY_NOTEBOOK_INSERT_GROUPED : EPHY_NOTEBOOK_INSERT_LAST;
 	ephy_notebook_insert_page (EPHY_NOTEBOOK (window->priv->notebook),
-				  widget,
-				  EPHY_NOTEBOOK_INSERT_GROUPED,
-				  jump_to);
+				   widget, position, jump_to);
 }
 
 void
@@ -1172,6 +1207,37 @@ update_find_control (EphyWindow *window)
 	}
 }
 
+static void
+update_tabs (EphyWindow *window)
+{
+	gboolean prev_tab, next_tab, move_left, move_right, detach;
+	EggActionGroup *action_group;
+	EggAction *action;
+	int current;
+	int last;
+
+	current = gtk_notebook_get_current_page
+		(GTK_NOTEBOOK (window->priv->notebook));
+	last = gtk_notebook_get_n_pages
+		(GTK_NOTEBOOK (window->priv->notebook)) - 1;
+	prev_tab = move_left = (current > 0);
+	next_tab = move_right = (current < last);
+	detach = gtk_notebook_get_n_pages
+		(GTK_NOTEBOOK (window->priv->notebook)) > 1;
+
+	action_group = window->priv->action_group;
+	action = egg_action_group_get_action (action_group, "TabsPrevious");
+	g_object_set (action, "sensitive", prev_tab, NULL);
+	action = egg_action_group_get_action (action_group, "TabsNext");
+	g_object_set (action, "sensitive", next_tab, NULL);
+	action = egg_action_group_get_action (action_group, "TabsMoveLeft");
+	g_object_set (action, "sensitive", move_left, NULL);
+	action = egg_action_group_get_action (action_group, "TabsMoveRight");
+	g_object_set (action, "sensitive", move_right, NULL);
+	action = egg_action_group_get_action (action_group, "TabsDetach");
+	g_object_set (action, "sensitive", detach, NULL);
+}
+
 static void
 update_window_visibility (EphyWindow *window)
 {
@@ -1227,6 +1293,9 @@ ephy_window_update_control (EphyWindow *window,
 
 	switch (control)
 	{
+	case TabsControl:
+		update_tabs (window);
+		break;
 	case StatusbarMessageControl:
 		update_status_message (window);
 		break;
@@ -1286,6 +1355,7 @@ ephy_window_update_all_controls (EphyWindow *window)
 		update_security (window);
 		update_find_control (window);
 		update_spinner_control (window);
+		update_tabs (window);
 	}
 }
 
@@ -1350,8 +1420,6 @@ update_embed_dialogs (EphyWindow *window,
 {
 	EphyEmbed *embed;
 	EphyDialog *find_dialog = window->priv->find_dialog;
-	EphyDialog *history_dialog = window->priv->history_dialog;
-	EphyDialog *history_sidebar = window->priv->history_sidebar;
 
 	embed = ephy_tab_get_embed (tab);
 
@@ -1361,20 +1429,6 @@ update_embed_dialogs (EphyWindow *window,
 			(EPHY_EMBED_DIALOG(find_dialog),
 			 embed);
 	}
-
-	if (history_dialog)
-	{
-		ephy_embed_dialog_set_embed
-			(EPHY_EMBED_DIALOG(history_dialog),
-			 embed);
-	}
-
-	if (history_sidebar)
-	{
-		ephy_embed_dialog_set_embed
-			(EPHY_EMBED_DIALOG(history_sidebar),
-			 embed);
-	}
 }
 
 static void
@@ -1441,30 +1495,14 @@ ephy_window_get_find_dialog (EphyWindow *window)
 	return dialog;
 }
 
-void
-ephy_window_show_history (EphyWindow *window)
-{
-	EphyEmbed *embed;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	window->priv->history_dialog = history_dialog_new_with_parent
-						(GTK_WIDGET(window),
-						 embed,
-						 FALSE);
-	g_object_add_weak_pointer
-                        (G_OBJECT(window->priv->history_dialog),
-                         (gpointer *)&window->priv->history_dialog);
-
-	ephy_dialog_show (window->priv->history_dialog);
-}
-
 void
 ephy_window_set_zoom (EphyWindow *window,
 		      gint zoom)
 {
 	EphyEmbed *embed;
+	gboolean zoom_out, zoom_normal;
+	EggActionGroup *action_group;
+	EggAction *action;
 
         g_return_if_fail (IS_EPHY_WINDOW (window));
 
@@ -1472,6 +1510,14 @@ ephy_window_set_zoom (EphyWindow *window,
         g_return_if_fail (embed != NULL);
 
         ephy_embed_zoom_set (embed, zoom, TRUE);
+
+	zoom_normal = (zoom != 100);
+	zoom_out = (zoom > 0);
+	action_group = window->priv->action_group;
+	action = egg_action_group_get_action (action_group, "ViewZoomOut");
+	g_object_set (action, "sensitive", zoom_out, NULL);
+	action = egg_action_group_get_action (action_group, "ViewZoomNormal");
+	g_object_set (action, "sensitive", zoom_normal, NULL);
 }
 
 Toolbar *
