@@ -543,9 +543,6 @@ session_save (Session *session,
 static void
 parse_embed (xmlNodePtr child, EphyWindow *window)
 {
-	EphyTab *tab;
-	EphyEmbed *embed;
-
 	while (child != NULL)
 	{
 		if (strcmp (child->name, "embed") == 0)
@@ -558,14 +555,10 @@ parse_embed (xmlNodePtr child, EphyWindow *window)
 			url = xmlGetProp (child, "url");
 			title = xmlGetProp (child, "title");
 
-			tab = ephy_tab_new ();
-		        embed = ephy_tab_get_embed (tab);
-
-			gtk_widget_show (GTK_WIDGET(embed));
-
-			ephy_window_add_tab (window, tab, FALSE);
-
-			ephy_embed_load_url (embed, url);
+			ephy_shell_new_tab (ephy_shell, window, NULL, url,
+					    EPHY_NEW_TAB_IN_EXISTING_WINDOW |
+					    EPHY_NEW_TAB_DONT_JUMP_TO |
+					    EPHY_NEW_TAB_APPEND_LAST);
 
 			xmlFree (url);
 			xmlFree (title);
