@@ -36,7 +36,10 @@
 #include "ephy-new-bookmark.h"
 #include "egg-toggle-action.h"
 #include "egg-editable-toolbar.h"
+#include "egg-toolbar-editor.h"
+#include "ephy-file-helpers.h"
 #include "toolbar.h"
+#include "ephy-state.h"
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -45,6 +48,7 @@
 #include <libgnomeui/gnome-about.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnome/gnome-help.h>
+#include <libgnome/gnome-program.h>
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkeditable.h>
 
@@ -243,7 +247,7 @@ void
 window_cmd_go_bookmarks (EggAction *action,
 			 EphyWindow *window)
 {
-	ephy_shell_show_bookmarks_editor (ephy_shell);
+	ephy_shell_show_bookmarks_editor (ephy_shell, GTK_WIDGET (window));
 }
 
 void
@@ -279,8 +283,10 @@ window_cmd_file_bookmark_page (EggAction *action,
 		(EPHY_NEW_BOOKMARK (new_bookmark), title);
 	ephy_new_bookmark_set_icon
 		(EPHY_NEW_BOOKMARK (new_bookmark), icon);
-	gtk_dialog_run (GTK_DIALOG (new_bookmark));
-	gtk_widget_destroy (new_bookmark);
+	g_signal_connect (G_OBJECT (new_bookmark), "response",
+			  G_CALLBACK (ephy_new_bookmark_response_cb),
+			  NULL);
+	gtk_widget_show (new_bookmark);
 }
 
 void
@@ -309,7 +315,7 @@ window_cmd_file_open (EggAction *action,
 		(single, wmain,
 		 _("Select the file to open"),
                  dir, NULL, modeOpen,
-                 &file, NULL, NULL, NULL);
+                 &file, NULL, NULL);
 
 	if (result == G_OK)
 	{
@@ -478,6 +484,25 @@ window_cmd_edit_find_prev (EggAction *action,
 	ephy_window_update_control (window, FindControl);
 }
 
+void
+window_cmd_view_bookmarks_bar (EggAction *action,
+			       EphyWindow *window)
+{
+	EmbedChromeMask mask;
+	gboolean active;
+	gboolean current_state;
+
+	mask = ephy_window_get_chrome (window);
+	active = EGG_TOGGLE_ACTION (action)->active;
+	current_state = (mask & EMBED_CHROME_BOOKMARKSBARON) > 0;
+
+	if (active != current_state)
+	{
+		mask ^= EMBED_CHROME_BOOKMARKSBARON;
+		ephy_window_set_chrome (window, mask);
+	}
+}
+
 void
 window_cmd_view_toolbar (EggAction *action,
 			 EphyWindow *window)
@@ -591,7 +616,7 @@ void
 window_cmd_go_history (EggAction *action,
 		       EphyWindow *window)
 {
-	ephy_window_show_history (window);
+	ephy_shell_show_history_window (ephy_shell, GTK_WIDGET (window));
 }
 
 void
@@ -619,19 +644,65 @@ window_cmd_edit_prefs (EggAction *action,
 	gtk_widget_show (GTK_WIDGET(dialog));
 }
 
+static void
+toolbar_editor_destroy_cb (GtkWidget *tbe,
+			   Toolbar *t)
+{
+	GtkWidget *window;
+
+	window = gtk_widget_get_toplevel (GTK_WIDGET (t));
+
+	egg_editable_toolbar_set_edit_mode (EGG_EDITABLE_TOOLBAR (t), FALSE);
+	ephy_window_update_all_controls (EPHY_WINDOW (window));
+}
+
+static void
+toolbar_editor_help_cb (GtkDialog          *dialog,
+		    	gint                response_id,
+		    	EggToolbarEditor   *editor)
+{
+	switch (response_id)
+	{
+    	case GTK_RESPONSE_HELP:
+		/* FIXME: Connect toolbar editor help */
+		break;
+	}
+}
+
 void
 window_cmd_edit_toolbar (EggAction *action,
 			 EphyWindow *window)
 {
-	Toolbar *toolbar;
+	GtkWidget *editor;
+	EphyToolbarsModel *model;
+	Toolbar *t;
 
-	toolbar = ephy_window_get_toolbar (window);
+	model = ephy_shell_get_toolbars_model (ephy_shell);
+	t = ephy_window_get_toolbar (window);
 
-	egg_editable_toolbar_edit (EGG_EDITABLE_TOOLBAR (toolbar),
-				   GTK_WIDGET (window));
+	editor = egg_toolbar_editor_new
+		(GTK_WINDOW (window),
+		 EGG_MENU_MERGE (window->ui_merge),
+		 EGG_TOOLBARS_MODEL (model));
+	gtk_dialog_add_button (GTK_DIALOG (editor),
+			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+	ephy_state_add_window (editor,
+			       "toolbar_editor",
+		               500, 330);
+	g_signal_connect (editor, "destroy",
+			  G_CALLBACK (toolbar_editor_destroy_cb),
+			  t);
+	g_signal_connect (G_OBJECT (editor), "response",
+			  G_CALLBACK (toolbar_editor_help_cb),
+			  editor);
+	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
+					 ephy_file ("epiphany-toolbar.xml"));
+	gtk_widget_show (editor);
+
+	egg_editable_toolbar_set_edit_mode (EGG_EDITABLE_TOOLBAR (t), TRUE);
 }
 
-void 
+void
 window_cmd_help_contents (EggAction *action,
 			 EphyWindow *window)
 {
@@ -661,10 +732,12 @@ window_cmd_help_contents (EggAction *action,
 
 void
 window_cmd_help_about (EggAction *action,
-		       EphyWindow *window)
+		       GtkWidget *window)
 {
 	static GtkWidget *about = NULL;
 	GdkPixbuf *icon;
+	const char *icon_path;
+	GdkPixbuf *logo;
 
 	static gchar *authors[] = {
 		"Marco Pesenti Gritti <mpeseng@tin.it>",
@@ -686,6 +759,11 @@ window_cmd_help_about (EggAction *action,
 		return;
 	}
 
+	icon_path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
+					  "epiphany.png", TRUE, NULL);
+	logo = gdk_pixbuf_new_from_file (icon_path, NULL);
+	g_return_if_fail (logo != NULL);
+
 	about = gnome_about_new(
 		       "Epiphany", VERSION,
 		       "Copyright \xc2\xa9 2002-2003 Marco Pesenti Gritti",
@@ -693,7 +771,9 @@ window_cmd_help_about (EggAction *action,
 		       (const char **)authors,
 		       (const char **)documenters,
 		       strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-		       NULL);
+		       logo);
+
+	g_object_unref (logo);
 
 	gtk_window_set_transient_for (GTK_WINDOW (about),
 				      GTK_WINDOW (window));
@@ -770,6 +850,8 @@ window_cmd_tabs_move_left  (EggAction *action,
 		child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), page);
 		gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), child, page - 1);
 	}
+
+	ephy_window_update_control (window, TabsControl);
 }
 
 void window_cmd_tabs_move_right (EggAction *action,
@@ -790,6 +872,8 @@ void window_cmd_tabs_move_right (EggAction *action,
 		child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), page);
 		gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), child, page + 1);
 	}
+
+	ephy_window_update_control (window, TabsControl);
 }
 
 void
